/*   -*- c -*-
 * 
 *  ----------------------------------------------------------------------
 *  Cool word distribution calculator.  Main module.
 *  ----------------------------------------------------------------------
 *  Created      : Sun Aug  4 14:12:12 1996 rinnet
 *  Last modified: Thu Nov 27 07:55:25 1997 tri
 *  ----------------------------------------------------------------------
 *  Copyright  1996-1997
 *  Timo J. Rinne <tri@iki.fi>
 * 
 *  Address: Cirion oy, PO-BOX 250, 00121 Helsinki, Finland
 *  ----------------------------------------------------------------------
 *  Any express or implied warranties are disclaimed.  In no event
 *  shall the author be liable for any damages caused (directly or
 *  otherwise) by the use of this software.
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "worddist.h"

word_atom_t allocate_new_word(char *word)
{
    word_atom_t wd;
    
    if (!word)
	return ((word_atom_t)0);
    wd = calloc(1, sizeof (*wd));
    if (!wd)
	return ((word_atom_t)0);
    wd->word = malloc(strlen(word) + 1);
    if (!wd->word) {
	free(wd);
	return ((word_atom_t)0);
    }
    strcpy(wd->word, word);
    wd->l = wd->r = ((word_atom_t)0);
    wd->cnt = 0;
    return wd;
}

void free_word(word_atom_t wd)
{
    if (wd && wd->word)
	free(wd->word);
    if (wd)
	free(wd);
    return;
}

void free_tree(word_atom_t *tree)
{
    if (!tree || !(*tree))
	return;
    if ((*tree)->l)
	free_tree(&((*tree)->l));
    if ((*tree)->r)
	free_tree(&((*tree)->r));
    free_word(*tree);
    *tree = (word_atom_t)0;
    return;
}

int str_case_cmp(char *s1, char *s2, int ign_case)
{
    if (ign_case) {
	while (tolower(*s1) == tolower(*s2)) {
	    if (!(*s1))
		return 0;
	    s1++;
	    s2++;
	}
	return ((tolower(*s1) < tolower(*s2)) ? -1 : 1);
    }
    while (*s1 == *s2) {
	if (!(*s1))
	    return 0;
	s1++;
	s2++;
    }
    return ((*s1 < *s2) ? -1 : 1);
    /*NOTREACHED*/
}

int add_word_to_tree(word_atom_t *tree, int *items, char *word, int ign_case)
{
    word_atom_t wd;
    int r;

    if ((!word) || (!tree))
	return 0;
    if (!(*tree)) { /* Empty tree */
	wd = allocate_new_word(word);
	if (!wd)
	    return 0;
	wd->cnt = 1;
	*tree = wd;
	if (items)
	    *items = 1;
	return 1;
    }
    wd = *tree;
    while(1) {
	r = str_case_cmp(wd->word, word, ign_case);
	if (r == 0) {
	    wd->cnt++;
	    if (items)
		(*items)++;
	    return 1;
	} else if (r > 0) {
	    if (!wd->l) {
		wd->l = allocate_new_word(word);
		if (!wd->l)
		    return 0;
		wd->l->cnt = 1;
		if (items)
		    (*items)++;
		return 1;
	    } else {
		wd = wd->l;
	    }
	} else { /* r < 0 */
	    if (!wd->r) {
		wd->r = allocate_new_word(word);
		if (!wd->r)
		    return 0;
		wd->r->cnt = 1;
		if (items)
		    (*items)++;
		return 1;
	    } else {
		wd = wd->r;
	    }
	}
    }
    /*NOTREACHED*/
}

int legal_word_char(int c)
{
    return(c && (isprint(c)) && (!(isspace(c))) && (!(ispunct(c))));
}

int add_line_to_tree(word_atom_t *tree, int *items, char *buf, int ign_case)
{
    char *hlp1, *hlp2;

    if ((!buf) || (!tree))
	return 0;
    hlp1 = hlp2 = buf;
    while(1) {
	while (!legal_word_char(*hlp1)) {
	    if (!*hlp1)
		return 1;
	    hlp1++;
	    hlp2++;
	}
	while (legal_word_char(*hlp2))
	    hlp2++;
	if (*hlp2) {
	    *hlp2 = '\000';
	    if (!add_word_to_tree(tree, items, hlp1, ign_case))
		return 0;
	    hlp2++;
	    hlp1 = hlp2;
	} else {
	    if (!add_word_to_tree(tree, items, hlp1, ign_case))
		return 0;
	    return 1;
	}
    }
    /*NOTREACHED*/
}

int add_file_to_tree(word_atom_t *tree, int *items, FILE *f, int ign_case)
{
    char buf[256];
    
    while (fgets(buf, sizeof (buf), f))
	if (!add_line_to_tree(tree, items, buf, ign_case))
	    return 0;
    return 1;
}

int print_tree(word_atom_t *tree, int items, int prop)
{
    int r = 0;

    if (!tree || !(*tree))
	return 0;
    if ((*tree)->l)
	r += print_tree(&((*tree)->l), items, prop);
    if (prop)
	printf("%s\t%3.3f%%\n", 
	       (*tree)->word, 
	        ((float)(*tree)->cnt) / ((float)items) * 100.0);
    else
	printf("%s\t%d\n", (*tree)->word, (*tree)->cnt);
    r++;
    if ((*tree)->r)
	r += print_tree(&((*tree)->r), items, prop);
    return r;
}

int main(int argc, char **argv)
{
    int i, c;
    word_atom_t tree = (word_atom_t)0;
    int items = 0;
    FILE *f;
    extern char *optarg;
    extern int optind;
    /* Following can be set with command line flags */
    int prop = 0, total = 0, ign_case = 0; 

    while ((c = getopt(argc, argv, "pPtTiI")) != -1) {
	switch (c) {
	case 'p':
	    prop = 1;
	    break;
	case 'P':
	    prop = 0;
	    break;
	case 't':
	    total = 1;
	    break;
	case 'T':
	    total = 0;
	    break;
	case 'i':
	    ign_case = 1;
	    break;
	case 'I':
	    ign_case = 0;
	    break;
	default:
	    fprintf(stderr, 
		    "USAGE: wordcount [-p|P] [-t|P] [-i|I] [file...]\n");
	    exit(1);
	}
    }
    if (argc > optind) {
	for (i = optind; i < argc; i++) {
	    if (f = fopen(argv[i], "r")) {
		if (!add_file_to_tree(&tree, &items, f, ign_case)) {
		    fclose(f);
		    fprintf(stderr, "Error reading file \"%s\".  Abort.\n",
			    argv[i]);
		    exit(1);
		}
		fclose(f);
	    } else {
		fprintf(stderr, "Can't open file \"%s\".  Skipped.\n", 
			argv[i]);
	    }
	}
    } else {
	if (!add_file_to_tree(&tree, &items, stdin, ign_case)) {
	    fprintf(stderr, "Error reading stdin.  Abort.\n");
	    exit(1);
	}
    }
    c = print_tree(&tree, items, prop);
    if (total)
	printf("TOTAL\t%d words (%d unique)\n", items, c);
    free_tree(&tree); /* Not needed, but shows that our tree is in order. */
    exit(0);
    /*NOTREACHED*/
}
/* eof (worddist.c) */
