/*   -*- c -*-
 * 
 *  ----------------------------------------------------------------------
 *  Cool word distribution calculator.  Private header.
 *  ----------------------------------------------------------------------
 *  Created      : Sun Aug  4 14:13:47 1996 rinnet
 *  Last modified: Sun Aug 25 23:17:27 1996 tri
 *  ----------------------------------------------------------------------
 *  Copyright  1996
 *  Timo J. Rinne <tri@iki.fi>
 * 
 *  Address: Cirion oy, PO-BOX 250, 00121 Helsinki, Finland
 *  ----------------------------------------------------------------------
 *  Any express or implied warranties are disclaimed.  In no event
 *  shall the author be liable for any damages caused (directly or
 *  otherwise) by the use of this software.
 */
#ifndef _WORDDIST_H_
#define _WORDDIST_H_ 1

typedef struct word_atom {
    struct word_atom *l;	/* left leaf   */
    struct word_atom *r;	/* right leaf  */
    char             *word;	/* word        */
    unsigned int      cnt;	/* counter     */
} word_atom, *word_atom_t;

/*
 * Dynamically allocate new struct word_atom and store word into it.
 */
word_atom_t allocate_new_word(char *word);
/*
 * Dynamically free struct word_atom and it's contents.
 */
void free_word(word_atom_t wd);
/*
 * Same as strcmp(3) but ignores case if ign_case != 0.
 */
int str_case_cmp(char *s1, char *s2, int ign_case);
/*
 * Dynamically free binary tree.  Root of the tree is *tree.
 * After the operation *tree is modified to NULL.
 */
void free_tree(word_atom_t *tree);
/*
 * Add word to binary tree.  Root of the tree is *tree.  If *tree 
 * is NULL new root is allocated and *tree is modified to point 
 * to the new root node.  Argument *item is incremented.
 */
int add_word_to_tree(word_atom_t *tree, int *items, char *word, int ign_case);
/*
 * Add string containing words to the tree.  Number of added
 * words is added to *items.
 */
int add_line_to_tree(word_atom_t *tree, int *items, char *buf, int ign_case);
/*
 * Add file pointed by *f containing words to the tree.  Number of 
 * added words is added to *items.
 */
int add_file_to_tree(word_atom_t *tree, int *items, FILE *f, int ign_case);
/*
 * Print contents of the tree.  If prop is nonzero percents are
 * calculated instead of absolute counts of various words.
 */
int print_tree(word_atom_t *tree, int items, int prop);
/*
 * Test if character c is something that is in the word or is it
 * nonprintable, white-space or punctuation character.
 */
int legal_word_char(int c);

#endif /* ! _WORDDIST_H_ */
/* eof (worddist.h) */
